/**
 * charactron.h -- structures and values for the charactron font-drawing
 * system
 *
 * Tyler Montbriand, 2008
 * tsm@accesscomm.ca
 * Released under the GNU GPL
 */

#ifndef __CHARACTRON_H__
#define __CHARACTRON_H__

#include "unifont.h"
#include "SDL_mmap.h"

#define CHARACTRON_FG 1
#define CHARACTRON_BG 0

/*#ifdef SCALE
#ifdef _WIN32_WCE
#define SCALE_N		2
#define SCALE_D		3
#else
#define SCALE_N		3
#define SCALE_D		4
#endif
#endif*/

typedef struct charactron
{
  unifont *rawfont;
  SDL_Surface *font;    /* 1 x 65536 grid of unicode characters */
  SDL_Surface *output;  /* surface that is drawn on             */

#ifdef SCALE
  SDL_Surface *scale;
#endif

  Sint8 dirx;		/* 0 for LTR, -1 for RTL		*/
  Sint8 xfree;
  Sint16 x,y;           /* position of x and y cursor           */
  Sint16 maxx,maxy;     /* width and height of output           */

  Uint32 fg,bg;         /* Cached fg/bg colors for filling      */
  Uint32 flag;
} charactron;

#define CH_ENABLE               0x80000000

#define CHARACTRON_FONT_W(c)	8
#define CHARACTRON_FONT_H(c)	16
#define CHARACTRON_H_FREE(c)    ((c)->maxy - (c)->y)
#define CHARACTRON_X_OFF(c)	(1+((c)->dirx<<1))

#define CHARACTRON_N_FREE(c)	((CHARACTRON_H_FREE(c) * (c)->maxx)+(c)->xfree)

#define CHARACTRON_X_RESET(c)	do	{		\
	(c)->x=((c)->maxx + (c)->dirx)%((c)->maxx);	\
	(c)->xfree=(c)->maxx;	} while(0)

#define CHARACTRON_SPACE(c)	do	{	\
	(c)->x+=CHARACTRON_X_OFF(c);		\
	(c)->xfree--;				\
	} while(0)


#define CHARACTRON_ENABLE(c)	do { (c)->flag |= CH_ENABLE; } while(0)
#define CHARACTRON_DISABLE(c)	do { (c)->flag &= ~CH_ENABLE; } while(0)

#define CHARACTRON_ENABLED(c)   ((c)->flag & CH_ENABLE)

#ifdef SCALE
void charactron_scale(charactron *c);
#else
#define charactron_scale(c)
#endif

void charactron_free(charactron *c);
charactron *charactron_load(unifont *font, SDL_Surface *out, int autoclose);

/* 'letter' designates a character or half-character.  Narrow characters 
 * are described by one character, numerated at double the unicode 
 * value.  Wide characters take two characters, the first at double the 
 * unicode value, the second at double the unicode value plus one.
 */
int charactron_draw_letter(charactron *c, Uint32 letter);
int charactron_draw_data(charactron *c, const Uint8 *mem, 
	mempos_t *start, mempos_t end);

//int charactron_draw_udata(charactron *c, const Uint32 *str, long len);
int charactron_linefeed(charactron *c);
int charactron_formfeed(charactron *c);

int charactron_set_color(charactron *c, int i, SDL_Color cl);

#endif/*__CHARACTRON_H__*/
