/**
 * The structure of a large flat file containing bitmap unicode font
 * characters derived from the GNU Unifont.
 *
 * Tyler Montbriand, 2008
 * tsm@accesscomm.ca
 * Released under the GNU GPL
 */
#ifndef __UNIFONT_H__
#define __UNIFONT_H__

#include <SDL/SDL_types.h>

#define CHARSET_SIZE    65536
#define CHAR_W		16
#define CHAR_H		16

#define CHARS_X		256
#define CHARS_Y		(CHARSET_SIZE/CHARS_X)

#define U_IMG_W		CHARS_X
#define U_IMG_H		((CHAR_W/8)*256*CHAR_H)
#define U_IMG_PITCH	CHARS_X
#define U_IMG_BPP	1

typedef struct unifont
{
        Uint8 glyphsize[CHARSET_SIZE/8];
	Uint8 encoded[CHARSET_SIZE/8];

	Uint8 charset_l[CHARS_Y*CHAR_H][CHARS_X];
	Uint8 charset_h[CHARS_Y*CHAR_H][CHARS_X];
} unifont;

#define BIT(N)  (1<<(N))

#define U_ROW(G)	(((G)/256)*CHAR_H)
#define U_COL(G)	((G)%256)

#define L_BYTE(U, G, N)	(U)->charset_l[U_ROW(G)+(N)][U_COL(G)]
#define H_BYTE(U, G, N)	(U)->charset_h[U_ROW(G)+(N)][U_COL(G)]

#define UNIFONT_WIDE(U, G)	((U)->glyphsize[(G)/8]&BIT((G)&7))
#define UNIFONT_ENCODED(U, G)	((U)->encoded[(G)/8]&BIT((G)&7))

#ifdef __cplusplus
extern "C" {
#endif

unifont *unifont_load(void);

#ifdef __cplusplus
}
#endif

#endif/*__UNIFONT_H__*/
