/**
 * Attempts to automatically map unifont.bin into memory
 *
 * Tyler Montbriand, 2008
 * tsm@accesscomm.ca
 * Released under the GNU GPL
 */
#include <stdio.h>
#include "unifont.h"
#include "SDL_mmap.h"

static SDL_mspan *uf_mm=NULL;

// Array of path locations to look for unifont.bin in
static const char *unifont_location[]={
#	ifdef UNIFONT_BASE
		UNIFONT_BASE,
#	endif

#	ifdef WIN32
		"c:\\unifont.bin",
		"..\\unifont.bin",
#	elif defined WIN32_WCE
		"/unifont.bin",
#	elif defined(LINUX)||defined(UNIX)||defined(__LINUX__)||defined(__UNIX__)
		"/usr/share/SDL_reader/unifont.bin",
#	endif
		"unifont.bin",
		NULL	};

unifont *unifont_load(void)
{
	int n;
	if(uf_mm == NULL)
	{
		for(n=0; unifont_location[n] != NULL; n++)
		{
			uf_mm=SDL_mmap_mapfile(unifont_location[n], SDL_MMAP_RD,
				SDL_MADVISE_RANDOM);

			if(uf_mm != NULL)
				break;
		}

		if(uf_mm == NULL)
        {
			fprintf(stderr, "Couldn't find or open unifont.bin\n");
			return(NULL);
        }
	}

	return((unifont *)(uf_mm->mem));
}

void unifont_free(void)
{
	if(uf_mm != NULL)
	{
		SDL_mmap_freefile(uf_mm);
		uf_mm=NULL;
	}
}
