/**
 * utf8.c -- UTF8 text stream parser, sort of a getc for unicode characters.
 *
 * Tyler Montbriand, 2008
 * tsm@accesscomm.ca
 * This portion is freeware.
 */
#include <stdio.h>
#include <string.h>
#include "utf8.h"

extern Uint32 octet_masks[], octet_res[];

// Nonzero if byte-string X matches the given length of UTF8 octet group
#define OCTET_MATCH(X, N)	(((X)&octet_masks[N])==octet_res[N])
#define OCTET_GET(X, N)		((X)>>((N)*8))

int utf8_getchar(const Uint8 *buf, Uint32 *pos_in, Uint32 maxpos)
{
	int tries=0;
	Uint32 pos;
	int octet=0;
	Uint32 acc=0;

UTF8_RESTART:
	pos=(*pos_in)+tries;

	while(pos<maxpos)
	{
		acc<<=8;
		acc|=(Uint32)buf[pos++];

		if(OCTET_MATCH(acc, octet))
		{
			Uint32 out;
//			fprintf(stderr, "Match on octet %d\n", octet+1);

			if(octet == 0)
			{
				(*pos_in)=pos;
				return(acc);
			}

			out=OCTET_GET(acc, octet)&(63>>octet);
			octet--;

			while(octet >= 0)
			{
//				fprintf(stderr, "...");
				out<<=6;
				out|=OCTET_GET(acc, octet)&0x3f;
				octet--;
			}

			(*pos_in)=pos;
//			fprintf(stderr, "Got %d\n", out);
			if(out > 65535)
				fprintf(stderr, "WARNING octet=%d\n", out);

			switch(out)
			{
			case 0x00a0:	// Unicode's many NBSP's
			case 0x200b:
			case 0x200d:
			case 0xfeff:
				tries=pos-(*pos_in);
				acc=0;
				octet=0;				
				goto UTF8_RESTART;

			default:
				break;
			}

//			fprintf(stderr,"Got: U+0x%04x\n", out);
//			if((out >= 0x0300) && (out < 0x036f))
//				fprintf(stderr, "Combining char: U+%04x\n", out);

			return(out);
		}

		octet++;
		if(octet > 3)
			break;
	}

	// Start one further down, maybe move past error
	if(((*pos_in)+tries) < maxpos)
	{
		tries++;
//		fprintf(stderr, "Try %d\n", tries);
		octet=0;
		acc=0;
		goto UTF8_RESTART;		
	}

//	fprintf(stderr, "No byte\n");
	return(-1);
}

/*
int main(void)
{
	const unsigned char str[]={0xd7, 0x90, 0x10, 0x00};
	int pos=0;
	int max=strlen(str);
	int c=utf8_getchar(str, &pos, max);

	while(c >= 0)
	{
		printf("Got:  %d\n", c);
		c=utf8_getchar(str, &pos, max);
	}

	return(0);
}*/

Uint32 octet_masks[]={	0x00000080,	// One-byte octet
                   	0x0000e0c0,	// Two-byte octet
			0x00f0c0c0,	// Three-byte octet
			0xf8c0c0c0 };	// Four-byte octet

Uint32 octet_res[]={	0x00000000,
			0x0000c080,
			0x00e08080,
			0xf0808080	};
